/*	
			Datum.equ
			
	Author:	Jim Mensch
	Date:		10-25-97
	
	Description:This file contains equates and definitions for use with the Datum PCI card
			while they are universal they are specifically designed for PowerPC based
			Macintoshes
			
	Change History:
	
	10-25-97	First input into file
	
	
*/

#include <Types.h>

/*	The Datum card is a bare bones PCI card that contains no FCode and a minimal configuration ROM
	this means that the board will use a generic ID made up of the mfg id/subid fields. Also data
	is latched and accessed by direct reference to data pointed to by the PCI configuration register map
	
	currently BAR 0 will point to the offset to the data and latches the card is interested in.
	
	The data is broken up into two main types:
		Main clock register which can be latched and read by software only 
		Event registers which are latched either by software or external events
		the following data structure defines the Map of this RAM:
		
		struct {
			long		MainClockLatch;	// access this long once to latch the time in the main clock
			long		MainClockLowerData;	// data and status latched by access to MainClockLatch
			long		MainClockUpperData;	// data latched by access to MainClockLatch
			long		EventClockLatch;	// access this to manually latch event time
			long		EventClockLower;	// latched event data and status
			long		EventClockUpper;	// latched event data
			long		EventControl;	// controls the external latching of data
		}
		
		The lower data long contains the following bit fields
		
		0-19		sec data
		20-23		100 nanosec's
		24-27		status bits
		
		the upper data field contains the major time data in xxxxx
		
		in the event control register
		
		bit xxx	allows external event triggering
		bit xxx	places the register in capture/lockout mode
		
*/

// offsets into our data RAM (used this way because we have to create an address)
// these are offset of BAR0 register value
#define oMainClockLatch		0				// access this long once to latch the time in the main clock
#define oEventClockLatch	oMainClockLatch+4		// access to manually latch the event time
#define oReleaseCaptureLock	oEventClockLatch+4	// access to release the capture lockout
#define oRes1			oReleaseCaptureLock+4
#define oControlRegister	oRes1+4			// See definition of bit fields below
#define oACKRegister		oControlRegister+4	// Register contents undefined at this time
#define oInteruptMask		oACKRegister+4		// se bit definition below
#define oInteruptStatus		oInteruptMask+4		// see bit definition below
#define oMinorStrobeTime	oInteruptStatus+4		// undefined
#define oMajorStrobeTime	oMinorStrobeTime+4	// undefined
#define oRes2			oMajorStrobeTime+4	// reserved for future use
#define oRes3			oRes2+4			// reserved for future use
#define oMainClockMinor		oRes3+4			// main clock lower value
#define oMainClockMajor		oMainClockMinor+4		// main clock upper value
#define oEventClockMinor	oMainClockMajor+4		// event clock lower data
#define oEventClockMajor	oEventClockMinor+4	//

// bit fields used to mask off lower data information
#define fMicroData	%00000000000011111111111111111111
#define fNanoData		%00000000111100000000000000000000
#define fStatusBits	%00001111000000000000000000000000

// bit fields for the time.lower status bits
#define fTracking		0b00000001000000000000000000000000
#define fPhase		0b00000010000000000000000000000000
#define fFrequency	0b00000100000000000000000000000000
#define fReserved		0b00001000000000000000000000000000

// bit definitions for interupt status/mask register
#define INT_EVENT		(1<<0)
#define INT_HBEAT		(1<<1)
#define INT_STROBE	(1<<2)
#define INT_1PPS		(1<<3)
#define INT_DPA		(1<<4)
#define INT_UNUSED	(1<<5)
#define INT_HIRES		(1<<6)
#define INT_HIRES2	(1<<7)

// bit definitions for the ACKRegister
#define ACK_RCV		(1<<0)
#define ACK_SEND		(1<<7)

// bit fields used to mask off event control registers
// fExternalTrigger set to 1 for periodic events, 0 for externally triggered events
#define fExternalTrig	0b000000000000000000000000000000010
// fCaptureLock set to true to prohibit multiple latch prior to reads
#define fCaptureLock	0b000000000000000000000000000000001
// fFalling Edge 0 = Rising edge trigger 1 = Falling edge trigger
#define fFallingEdge	0b000000000000000000000000000000100
// fEventsOn Set to 1 to enable the event clocks updating
#define fEventsOn		0b000000000000000000000000000001000
// fTimeCoincidenceEnable 1 enables externale strobe output
#define fTimeCoincidenceEnable 0b000000000000000000000000000010000
// fTimeCoincidenceMode set to 0 use major/minor mode  1 is minor mode (every second pulse...)
#define fTimeCoincidenceMode 0b000000000000000000000000000100000
// fOutputFrequency Select 00 for 10Mhz 01 for 5Mhz  10 for 1Mhz 11 for other (currently 1Mhz)
#define fOutputFrequency 0b000000000000000000000000011000000

#define DatumBoardName	"pci12e2,4013"
#define GoofyTimeOffset 0x7c25b080

struct ClockData {
	UInt32		ClockLower;
	UInt32		ClockUpper;
	UInt32		StatusControl;
};
typedef struct ClockData ClockData, *ClockDataPtr;

struct PCIConfigHeader {
	UInt32			IDs;
	UInt32			StatusCommand;
	UInt32			ClassRev;
	UInt32			DevInfo;
	UInt32			BAR0;
	UInt32			BAR1;
	UInt32			BAR2;
	UInt32			BAR3;
	UInt32			BAR4;
	UInt32			BAR5;
	UInt32			CISPtr;
	UInt32			SubsysIDs;
	UInt32			ROMBase;
	UInt32			Reserved1;
	UInt32			Reserved2;
	UInt32			IntInfo;
};
typedef	struct	PCIConfigHeader PCIConfigHeader, *PCIConfigPtr;
	
struct BoardStatus {
	Boolean		installed;
	short		status;
	short		control;
};
typedef struct BoardStatus BoardStatus, *BoardStatusPtr;	


/*
	theFollowing Routines will be used to access the data from the clock
*/
	UInt32	DatumRead(UInt32	offset);		// use the above defined offset fields
									// to access raw data from the clock
	UInt32	DatumWrite(UInt32	offset, UInt32	data);	// use to set data in the clock
	
	UInt32	GetMainClock(ClockDataPtr	data);	// reads the main clock time into the structure
	UInt32	GetEventClock(ClockDataPtr	data);	// reads and resets the event clock
	UInt32	SetEventControl(UInt32		data);	// sets up the event clock control register
	UInt32	LatchMainClock();					// latches the current time into the main clock
	UInt32	LatchEventClock();				// latches the current time into the event clock
	void		ClearEventLock();
	
	UInt32	GetInteruptStatus();
	void		ResetInteruptStatus(UInt32 theData);
	void		SetBoardControl(UInt32 theData);
	UInt32	GetBoardControl();
	UInt32	GetInteruptMask();
	void		SetInteruptMask(UInt32 theData);
	
	UInt32	GetCardConfigRec(Ptr	theDataBlock);
	OSErr		GetClockStatus(BoardStatusPtr	theStatus);
	OSErr	 	ConnectionRtn();
	void		ConnectionBroke();


